@@ -1,7 +1,7 @@
-#' Make MA-List object 
+#' Make MA-List Object 
 #'
 #' \code{makeMAList} extracts sets of data-pairs (like R & G series) and makes MA objects as \code{MA-List object} (eg for ratio oriented analysis).
-#' The grouping of columns as sets of replicate-measurements is done according to argumnet \code{'MAfac'}.
+#' The grouping of columns as sets of replicate-measurements is done according to argumnet \code{MAfac}.
 #' The output is fully compatible to functions of package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} (Bioconductor). 
 #'
 #' @details This function requires Bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} being installed.
@@ -25,19 +25,21 @@ makeMAList <- function(mat, MAfac, useF=c("R","G"), isLog=TRUE, silent=FALSE, de
   ## 'MAfac' .. factor
   ## require(limma)
   fxNa <- .composeCallName(callFrom, newNa="makeMAList")
-  if(!isTRUE(silent)) silent <- FALSE
-  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
-    
-  chPa <- requireNamespace("limma", quietly=TRUE)
-  if(!chPa) {return(NULL); warning(fxNa,"Package 'limma' not found ! Please install from Bioconductor")
-  } else {
-    if(!all(useF %in% MAfac & length(useF) ==2 & length(MAfac) >1)) stop(" Argument 'useF' should describe 2 elements of 'MAfac'")
-    if(!isLog) {
-      if(any(mat <0) & !silent) message(fxNa,"Negative values will create NAs at log2-transformation !") }
-    out <- try(limma::MA.RG(if(isLog) list(R=2^mat[,which(MAfac==useF[1])], G=2^mat[,which(MAfac==useF[2])]) else {
-      list(R=mat[,which(MAfac==useF[1])], G=mat[,which(MAfac==useF[2])])}, bc.method="subtract", offset=0), silent=TRUE)
-    if(inherits(out, "try-error")) {warning("UNABLE to run limma::MA.RG() '!"); out <- NULL}
-    out } }
+  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
+    if(!isTRUE(silent)) silent <- FALSE }
+  if(length(mat) >0 && length(dim(mat)) >1) {  
+    if(requireNamespace("limma", quietly=TRUE)) {
+      if(!(all(useF %in% MAfac) && length(useF) ==2 && length(MAfac) >1)) stop(fxNa,"Bad input;  Argument 'useF' should describe 2 elements of 'MAfac'")
+      if(!isLog) {
+        if(any(mat <0) && !silent) message(fxNa,"Negative values will create NAs at log2-transformation !") }
+      out <- try(limma::MA.RG(if(isLog) list(R=2^mat[,which(MAfac==useF[1])], G=2^mat[,which(MAfac==useF[2])]) else {
+        list(R=mat[,which(MAfac==useF[1])], G=mat[,which(MAfac==useF[2])])}, bc.method="subtract", offset=0), silent=TRUE)
+      if(inherits(out, "try-error")) {warning(fxNa,"UNABLE to run limma::MA.RG() '!"); out <- NULL}
+      out
+    } else {
+      return(NULL); if(!silent) message(fxNa,"NOTE: Package 'limma' not found ! Please install from Bioconductor  (returning NULL)") }
+  } else if(!silent) message(fxNa,"Input is empty or invalid, returning NULL")
+    }
 
 
 #' Search character-string and cut either before or after 
@@ -89,15 +91,15 @@ makeMAList <- function(mat, MAfac, useF=c("R","G"), isLog=TRUE, silent=FALSE, de
   yDimNa <- dimnames(y)
   if(length(xDimNa) <1) x <- matrix(x, ncol=1, dimnames=list(rownames(x),"x1"))
   if(length(yDimNa) <1) y <- matrix(y, ncol=1, dimnames=list(rownames(y),"y1"))
-  if(nrow(x) != nrow(y)) stop(fxNa," number of rows in 'x' and 'y' must be equal !")
+  if(nrow(x) != nrow(y)) stop(fxNa,"Number of rows in 'x' and 'y' must be equal !")
   ## main
-  out <- apply(x, 2, function(da,re) matrix(rep(da,ncol(re)), ncol=ncol(re))/re, re=y)    # is this really ok ?
+  out <- apply(x, 2, function(da, re) matrix(rep(da,ncol(re)), ncol=ncol(re))/re, re=y)    # is this really ok ?
   if(!is.list(out)) out <- list(out)
   if(asLog2) out <- lapply(out,log2)
   out <- if(identical(sumMeth,"mean")) lapply(out,rowMeans) else lapply(out,function(x) apply(x, 1, stats::median,na.rm=TRUE))     # summarize
   nLen <- sapply(out,length)
-  if(length(unique(nLen)) >1 | any(nLen <1)) message(fxNa," strange format of results, length of lists: ",
-    paste(nLen,collapse=" "))
+  if(length(unique(nLen)) >1 || any(nLen <1)) message(fxNa,"Strange format of results, length of lists: ",
+    paste(nLen, collapse=" "))
   out <- out[which(nLen >0)]
   out <- if(length(out) >1) matrix(unlist(out), nrow=nrow(x), dimnames=xDimNa) else out[[1]]
   out }
@@ -116,8 +118,8 @@ makeMAList <- function(mat, MAfac, useF=c("R","G"), isLog=TRUE, silent=FALSE, de
 .getAmean <- function(dat, grp) {
   ## get A value (ie group mean) for each group of replicates (eg for MA-plot)
   ## NOTE : this fx is redundant; does about the same as .rowGrpMeans() !
-  if(length(levels(grp)) <2) stop(" problem: 'grp' as factor should have at least 2 levels")
-  if(length(grp) != ncol(dat)) stop(" problem: length of 'grp' should match numbe of columns in 'dat'")
+  if(length(levels(grp)) <2) stop("Problem: 'grp' as factor should have at least 2 levels")
+  if(length(grp) != ncol(dat)) stop("Problem: length of 'grp' should match numbe of columns in 'dat'")
   if(!is.factor(grp)) grp <- as.factor(grp)
   out <- matrix(nrow=nrow(dat), ncol=length(levels(grp)), dimnames=list(rownames(dat), levels(grp)))
   for(i in 1:length(levels(grp))) out[,i] <- rowMeans(dat[,which(grp==levels(grp)[i])],na.rm=TRUE)
